@@ -4,6 +4,7 @@ use crate::{codec::value::ValueError, shell::platform::error::PlatformError};
 
 #[derive(Debug, Clone)]
 pub enum Error {
+    InvalidContext,
     InvalidEngineHandle,
     Platform(PlatformError),
     Value(ValueError),
@@ -13,6 +14,9 @@ pub enum Error {
 impl Display for Error {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         match self {
+            Error::InvalidContext => {
+                write!(f, "Context was already destroyed")
+            }
             Error::Platform(error) => Display::fmt(error, f),
             Error::InvalidEngineHandle => {
                 write!(f, "Provided handle does not match any engine")
