@@ -5,9 +5,12 @@ mod context;
 mod engine;
 mod engine_manager;
 mod geometry;
+mod handle;
+mod keyboard_map_manager;
 mod macros;
 mod menu_manager;
 mod message_manager;
+mod method_channel;
 mod observatory;
 mod run_loop;
 mod window;
@@ -20,9 +23,12 @@ pub use context::*;
 pub use engine::*;
 pub use engine_manager::*;
 pub use geometry::*;
+pub use handle::*;
+pub use keyboard_map_manager::*;
 pub use macros::*;
 pub use menu_manager::*;
 pub use message_manager::*;
+pub use method_channel::*;
 pub use observatory::*;
 pub use run_loop::*;
 pub use window::*;
