@@ -6,7 +6,7 @@ use std::{
 };
 
 use super::flutter_sys;
-use glib::{glib_wrapper, Object};
+use glib::Object;
 use glib::{translate::*, Bytes};
 use gtk::{Container, Widget};
 
@@ -14,12 +14,12 @@ use glib::object::Cast;
 use glib::object::IsA;
 use glib::GString;
 
-glib_wrapper! {
+glib::wrapper! {
     pub struct DartProject(Object<flutter_sys::FlDartProject,
-        flutter_sys::FlDartProjectClass, DartProjectClass>);
+        flutter_sys::FlDartProjectClass>);
 
     match fn {
-        get_type => || gobject_sys::g_object_get_type(),
+        type_ => || gobject_sys::g_object_get_type(),
     }
 }
 
@@ -29,12 +29,12 @@ impl DartProject {
     }
 }
 
-glib_wrapper! {
+glib::wrapper! {
     pub struct View(Object<flutter_sys::FlView,
-        flutter_sys::FlViewClass, ViewClass>) @extends Container, Widget;
+        flutter_sys::FlViewClass>) @extends Container, Widget;
 
     match fn {
-        get_type => || gtk_sys::gtk_container_get_type(),
+        type_ => || gtk_sys::gtk_container_get_type(),
     }
 }
 
@@ -72,12 +72,12 @@ impl<O: IsA<View>> ViewExt for O {
     }
 }
 
-glib_wrapper! {
+glib::wrapper! {
     pub struct Engine(Object<flutter_sys::FlEngine,
-        flutter_sys::FlEngineClass, EngineClass>);
+        flutter_sys::FlEngineClass>);
 
     match fn {
-        get_type => || gobject_sys::g_object_get_type(),
+        type_ => || gobject_sys::g_object_get_type(),
     }
 }
 
@@ -96,21 +96,21 @@ impl<O: IsA<Engine>> EngineExt for O {
     }
 }
 
-glib_wrapper! {
+glib::wrapper! {
     pub struct BinaryMessenger(Object<flutter_sys::FlBinaryMessenger,
-        flutter_sys::FlBinaryMessengerClass, BinaryMessengerClass>);
+        flutter_sys::FlBinaryMessengerClass>);
 
     match fn {
-        get_type => || gobject_sys::g_object_get_type(),
+        type_ => || gobject_sys::g_object_get_type(),
     }
 }
 
-glib_wrapper! {
+glib::wrapper! {
     pub struct BinaryMessengerResponseHandle(Object<flutter_sys::FlBinaryMessengerResponseHandle,
-        flutter_sys::FlBinaryMessengerResponseHandleClass, BinaryMessengerResponseHandleClass>);
+        flutter_sys::FlBinaryMessengerResponseHandleClass>);
 
     match fn {
-        get_type => || gobject_sys::g_object_get_type(),
+        type_ => || gobject_sys::g_object_get_type(),
     }
 }
 
