@@ -23,7 +23,7 @@ use crate::{
     shell::api_model::{Menu, MenuItem, MenuItemRole},
     shell::{
         api_model::{Accelerator, CheckStatus, MenuRole},
-        Context, MenuHandle, MenuManager, ScheduledCallback,
+        Context, Handle, MenuHandle, MenuManager,
     },
     util::{update_diff, DiffResult, LateRefCell},
 };
@@ -50,14 +50,14 @@ impl Hash for StrongPtrWrapper {
 }
 
 pub struct PlatformMenuManager {
-    context: Rc<Context>,
+    context: Context,
     app_menu: RefCell<Option<Rc<PlatformMenu>>>,
     window_menus: RefCell<HashMap<StrongPtrWrapper, Rc<PlatformMenu>>>,
-    update_handle: RefCell<Option<ScheduledCallback>>,
+    update_handle: RefCell<Option<Handle>>,
 }
 
 impl PlatformMenuManager {
-    pub fn new(context: Rc<Context>) -> Self {
+    pub fn new(context: Context) -> Self {
         Self {
             context,
             app_menu: RefCell::new(None),
@@ -95,15 +95,19 @@ impl PlatformMenuManager {
     }
 
     fn schedule_update(&self) {
-        let context = self.context.clone();
-        let callback = self.context.run_loop.borrow().schedule_now(move || {
-            context
-                .menu_manager
-                .borrow()
-                .get_platform_menu_manager()
-                .update_menu();
-        });
-        self.update_handle.borrow_mut().replace(callback);
+        let context_clone = self.context.clone();
+        if let Some(context) = self.context.get() {
+            let callback = context.run_loop.borrow().schedule_now(move || {
+                if let Some(context) = context_clone.get() {
+                    context
+                        .menu_manager
+                        .borrow()
+                        .get_platform_menu_manager()
+                        .update_menu();
+                }
+            });
+            self.update_handle.borrow_mut().replace(callback);
+        }
     }
 
     pub fn set_app_menu(&self, menu: Option<Rc<PlatformMenu>>) -> PlatformResult<()> {
@@ -151,7 +155,7 @@ impl PlatformMenuManager {
 }
 
 pub struct PlatformMenu {
-    context: Rc<Context>,
+    context: Context,
     handle: MenuHandle,
     pub(super) menu: StrongPtr,
     previous_menu: RefCell<Menu>,
@@ -163,13 +167,16 @@ pub struct PlatformMenu {
 const ITEM_TAG: NSInteger = 9999;
 
 impl PlatformMenu {
-    pub fn new(context: Rc<Context>, handle: MenuHandle) -> Self {
+    pub fn new(context: Context, handle: MenuHandle) -> Self {
         unsafe {
             let menu: id = NSMenu::alloc(nil).initWithTitle_(*to_nsstring(""));
             let () = msg_send![menu, setAutoenablesItems: NO];
 
             let target: id = msg_send![MENU_ITEM_TARGET_CLASS.0, new];
             let target = StrongPtr::new(target);
+
+            let () = msg_send![menu, setDelegate:*target];
+
             Self {
                 context,
                 handle,
@@ -290,10 +297,13 @@ impl PlatformMenu {
             .iter()
             .filter_map(|f| f.submenu)
             .collect();
-        for c in children {
-            let menu = self.context.menu_manager.borrow().get_platform_menu(c);
-            if let Ok(menu) = menu {
-                menu.prepare_for_app_menu();
+
+        if let Some(context) = self.context.get() {
+            for c in children {
+                let menu = context.menu_manager.borrow().get_platform_menu(c);
+                if let Ok(menu) = menu {
+                    menu.prepare_for_app_menu();
+                }
             }
         }
     }
@@ -484,14 +494,22 @@ impl PlatformMenu {
     }
 
     fn menu_item_action(&self, item: id) {
-        let item_id = unsafe {
-            let object: id = msg_send![item, representedObject];
-            msg_send![object, longLongValue]
-        };
-        self.context
-            .menu_manager
-            .borrow()
-            .on_menu_action(self.handle, item_id);
+        if let Some(context) = self.context.get() {
+            let item_id = unsafe {
+                let object: id = msg_send![item, representedObject];
+                msg_send![object, longLongValue]
+            };
+            context
+                .menu_manager
+                .borrow()
+                .on_menu_action(self.handle, item_id);
+        }
+    }
+
+    fn on_menu_will_open(&self) {
+        if let Some(context) = self.context.get() {
+            context.menu_manager.borrow().on_menu_open(self.handle);
+        }
     }
 
     fn create_menu_item(&self, menu_item: &MenuItem, menu_manager: &MenuManager) -> StrongPtr {
@@ -529,6 +547,11 @@ lazy_static! {
             on_action as extern "C" fn(&Object, Sel, id),
         );
 
+        decl.add_method(
+            sel!(menuWillOpen:),
+            menu_will_open as extern "C" fn(&Object, Sel, id),
+        );
+
         MenuItemTargetClass(decl.register())
     };
 }
@@ -557,6 +580,17 @@ extern "C" fn on_action(this: &Object, _sel: Sel, sender: id) {
     }
 }
 
+extern "C" fn menu_will_open(this: &Object, _: Sel, _menu: id) {
+    let state_ptr = unsafe {
+        let state_ptr: *mut c_void = *this.get_ivar("imState");
+        &mut *(state_ptr as *mut Weak<PlatformMenu>)
+    };
+    let upgraded = state_ptr.upgrade();
+    if let Some(upgraded) = upgraded {
+        upgraded.on_menu_will_open();
+    }
+}
+
 #[allow(clippy::branches_sharing_code)]
 fn remove_mnemonics(title: &str) -> String {
     let mut res = String::new();
