@@ -5,8 +5,13 @@ pub mod engine;
 pub mod error;
 pub mod init;
 pub mod key_event;
+pub mod keyboard_map;
 pub mod menu;
 pub mod run_loop;
 mod utils;
 pub mod window;
-pub mod bundle;
\ No newline at end of file
+pub mod bundle;
+
+#[allow(dead_code)]
+#[allow(non_upper_case_globals)]
+mod keyboard_map_sys;
\ No newline at end of file
