@@ -3,6 +3,7 @@ use std::{
     collections::HashMap,
     ffi::c_void,
     rc::{Rc, Weak},
+    time::Duration,
 };
 
 use cocoa::{
@@ -10,7 +11,7 @@ use cocoa::{
         NSEvent, NSEventType, NSView, NSWindow, NSWindowCollectionBehavior, NSWindowStyleMask,
     },
     base::{id, nil, BOOL, NO, YES},
-    foundation::{NSArray, NSInteger, NSPoint, NSRect, NSSize, NSUInteger},
+    foundation::{NSArray, NSInteger, NSPoint, NSRect, NSSize, NSString, NSUInteger},
 };
 use cocoa::{
     appkit::{NSScreen, NSWindowTabbingMode},
@@ -54,7 +55,7 @@ use super::{
 pub type PlatformWindowType = StrongPtr;
 
 pub struct PlatformWindow {
-    context: Rc<Context>,
+    context: Context,
     platform_window: PlatformWindowType,
     parent_platform_window: Option<WeakPtr>,
     platform_delegate: StrongPtr,
@@ -75,7 +76,7 @@ extern "C" {
 
 impl PlatformWindow {
     pub fn new(
-        context: Rc<Context>,
+        context: Context,
         delegate: Weak<dyn PlatformWindowDelegate>,
         parent: Option<Rc<PlatformWindow>>,
     ) -> Self {
@@ -139,9 +140,11 @@ impl PlatformWindow {
             NSWindow::setContentSize_(*self.platform_window, Size::wh(1.0, 1.0).into());
         }
 
-        let drag_context = DragContext::new(self.context.clone(), weak);
-        drag_context.register(*self.platform_window);
-        self.drag_context.set(drag_context);
+        if let Some(context) = self.context.get() {
+            let drag_context = DragContext::new(&context, weak);
+            drag_context.register(*self.platform_window);
+            self.drag_context.set(drag_context);
+        }
     }
 
     pub fn get_platform_window(&self) -> PlatformWindowType {
@@ -421,67 +424,84 @@ impl PlatformWindow {
         }
     }
 
-    fn show_when_ready(weak_self: Weak<PlatformWindow>) {
+    fn show_when_ready(weak_self: Weak<PlatformWindow>, attempt: i32) {
         if let Some(s) = weak_self.upgrade() {
             autoreleasepool(|| unsafe {
-                // FIXME(knopp)
-                // This code used to check surface dimensions but it no longer works.
-                // Adding metal compositing in engine broke it; The patch was reverted
-                // in the meanwhile though. Update this once the dust settles.
-
-                // let layer = NSWindow::contentView(*s.platform_window).layer();
-                // let sublayers: id = msg_send![layer, sublayers];
-                // let first = sublayers.objectAtIndex(0);
-                // let contents: id = msg_send![first, contents];
-                // if contents != nil {
-                //     // This makes assumptions about FlutterView internals :-/
-                //     let class: id = msg_send![contents, className];
-                //     if !class.isEqualToString("IOSurface") {
-                //         panic!("Expected IOSurface content");
-                //     }
-                //     let scale = NSWindow::backingScaleFactor(*s.platform_window);
-                //     let content_size = NSView::frame(NSWindow::contentView(*s.platform_window));
-
-                //     let expected_width = scale * content_size.size.width;
-                //     let expected_height = scale * content_size.size.height;
-                //     // IOSurface width/height
-                //     let actual_width: NSInteger = msg_send![contents, width];
-                //     let actual_height: NSInteger = msg_send![contents, height];
-
-                //     // only show if size matches, otherwise we caught the view during resizing
-                //     if actual_width == expected_width as NSInteger
-                //         && actual_height == expected_height as NSInteger
-                //     {
-                s.actually_show();
-                if let Some(delegate) = s.delegate.upgrade() {
-                    delegate.visibility_changed(true);
+                let view = NSWindow::contentView(*s.platform_window);
+                let subviews: id = msg_send![view, subviews];
+                let view = {
+                    if subviews.count() > 0 {
+                        subviews.objectAtIndex(0)
+                    } else {
+                        view
+                    }
                 };
-                return;
-                // }
-                // }
-                // wait until we have content generated (with proper size)
-                // s.context
-                //     .run_loop
-                //     .borrow()
-                //     .schedule(Duration::from_secs_f64(1.0 / 60.0), move || {
-                //         Self::show_when_ready(weak_self)
-                //     })
-                //     .detach();
-            })
+
+                // If our assumptions about the layout below are wrong, don't keep
+                // waiting indefinitely.
+                let mut show = attempt == 5;
+
+                if !show {
+                    let layer = view.layer();
+                    let sublayers: id = msg_send![layer, sublayers];
+                    let first = sublayers.objectAtIndex(0);
+                    let contents: id = msg_send![first, contents];
+                    if contents != nil {
+                        // This makes assumptions about FlutterView internals :-/
+                        let class: id = msg_send![contents, className];
+                        if !class.isEqualToString("IOSurface") {
+                            panic!("Expected IOSurface content");
+                        }
+                        let scale = NSWindow::backingScaleFactor(*s.platform_window);
+                        let content_size = NSView::frame(NSWindow::contentView(*s.platform_window));
+
+                        let expected_width = scale * content_size.size.width;
+                        let expected_height = scale * content_size.size.height;
+                        // IOSurface width/height
+                        let actual_width: NSInteger = msg_send![contents, width];
+                        let actual_height: NSInteger = msg_send![contents, height];
+
+                        // only show if size matches, otherwise we caught the view during resizing
+                        if actual_width == expected_width as NSInteger
+                            && actual_height == expected_height as NSInteger
+                        {
+                            show = true;
+                        }
+                    }
+                }
+
+                if show {
+                    s.actually_show();
+                    if let Some(delegate) = s.delegate.upgrade() {
+                        delegate.visibility_changed(true);
+                    };
+                } else {
+                    if let Some(context) = s.context.get() {
+                        // wait until we have content generated (with proper size)
+                        context
+                            .run_loop
+                            .borrow()
+                            .schedule(Duration::from_secs_f64(1.0 / 60.0), move || {
+                                Self::show_when_ready(weak_self, attempt + 1)
+                            })
+                            .detach();
+                    }
+                }
+            });
         }
     }
 
     pub fn ready_to_show(&self) -> PlatformResult<()> {
         self.ready_to_show.set(true);
         if self.show_when_ready.get() {
-            Self::show_when_ready(self.weak_self.clone_value());
+            Self::show_when_ready(self.weak_self.clone_value(), 0);
         }
         Ok(())
     }
 
     pub fn show(&self) -> PlatformResult<()> {
         if self.ready_to_show.get() {
-            Self::show_when_ready(self.weak_self.clone_value());
+            Self::show_when_ready(self.weak_self.clone_value(), 0);
         } else {
             self.show_when_ready.set(true);
         }
@@ -671,7 +691,8 @@ impl PlatformWindow {
             // cocoa eats mouse up on popup menu
             self.synthetize_mouse_up_event();
 
-            let position: NSPoint = request.position.into();
+            let mut position: NSPoint = request.position.into();
+            flip_position(self.platform_window.contentView(), &mut position);
 
             let view = StrongPtr::retain(self.platform_window.contentView());
             let menu = menu.menu.clone();
@@ -700,7 +721,9 @@ impl PlatformWindow {
             //
             // Note there is a special support in MacOS PlatformRunLoop when scheduling tasks
             // run in 0 time to not block the dispatch queue.
-            self.context.run_loop.borrow().schedule_now(cb).detach();
+            if let Some(context) = self.context.get() {
+                context.run_loop.borrow().schedule_now(cb).detach();
+            }
         }
     }
 
@@ -717,11 +740,13 @@ impl PlatformWindow {
     }
 
     pub fn set_window_menu(&self, menu: Option<Rc<PlatformMenu>>) -> PlatformResult<()> {
-        self.context
-            .menu_manager
-            .borrow()
-            .get_platform_menu_manager()
-            .set_menu_for_window(self.platform_window.clone(), menu);
+        if let Some(context) = self.context.get() {
+            context
+                .menu_manager
+                .borrow()
+                .get_platform_menu_manager()
+                .set_menu_for_window(self.platform_window.clone(), menu);
+        }
         Ok(())
     }
 
@@ -901,35 +926,38 @@ extern "C" fn window_will_close(this: &Object, _: Sel, _: id) {
             }
             let () = msg_send![*state.platform_window, setContentViewController: nil];
         }
-        state
-            .context
-            .menu_manager
-            .borrow()
-            .get_platform_menu_manager()
-            .window_will_close(state.platform_window.clone());
+        if let Some(context) = state.context.get() {
+            context
+                .menu_manager
+                .borrow()
+                .get_platform_menu_manager()
+                .window_will_close(state.platform_window.clone());
+        }
         delegate.will_close();
     });
 }
 
 extern "C" fn window_did_become_key(this: &Object, _: Sel, _: id) {
     with_state_delegate(this, |state, _delegate| {
-        state
-            .context
-            .menu_manager
-            .borrow()
-            .get_platform_menu_manager()
-            .window_did_become_active(state.platform_window.clone());
+        if let Some(context) = state.context.get() {
+            context
+                .menu_manager
+                .borrow()
+                .get_platform_menu_manager()
+                .window_did_become_active(state.platform_window.clone());
+        }
     });
 }
 
 extern "C" fn window_did_resign_key(this: &Object, _: Sel, _: id) {
     with_state_delegate(this, |state, _delegate| {
-        state
-            .context
-            .menu_manager
-            .borrow()
-            .get_platform_menu_manager()
-            .window_did_resign_active(state.platform_window.clone());
+        if let Some(context) = state.context.get() {
+            context
+                .menu_manager
+                .borrow()
+                .get_platform_menu_manager()
+                .window_did_resign_active(state.platform_window.clone());
+        }
     });
 }
 
