@@ -9,9 +9,9 @@ use windows::create_instance;
 use crate::{
     shell::{
         api_model::{DragData, DragEffect, DragRequest, DraggingInfo},
-        Context, IPoint,
+        Context, ContextRef, IPoint,
     },
-    util::LateRefCell,
+    util::{LateRefCell, OkLog},
 };
 
 use super::{
@@ -24,14 +24,13 @@ use super::{
         create_dragimage_bitmap, CLSID_DragDropHelper,
     },
     error::PlatformResult,
-    util::HRESULTExt,
     window::PlatformWindow,
 };
 
 use super::all_bindings::*;
 
 pub struct DragContext {
-    context: Rc<Context>,
+    context: Context,
     weak_self: LateRefCell<Weak<DragContext>>,
     window: Weak<PlatformWindow>,
     drag_data: RefCell<Option<DragData>>,
@@ -40,9 +39,9 @@ pub struct DragContext {
 }
 
 impl DragContext {
-    pub fn new(context: Rc<Context>, window: Weak<PlatformWindow>) -> Self {
+    pub fn new(context: &ContextRef, window: Weak<PlatformWindow>) -> Self {
         Self {
-            context: context.clone(),
+            context: context.weak(),
             weak_self: LateRefCell::new(),
             window,
             drag_data: RefCell::new(None),
@@ -70,29 +69,33 @@ impl DragContext {
 
     pub fn shut_down(&self) -> PlatformResult<()> {
         let window = self.window.upgrade().unwrap();
-        unsafe { RevokeDragDrop(window.hwnd()).as_platform_result() }
+        unsafe { RevokeDragDrop(window.hwnd()).map_err(|e| e.into()) }
     }
 
     pub fn begin_drag_session(&self, request: DragRequest) -> PlatformResult<()> {
         let weak = self.weak_self.clone_value();
-        self.context
-            .run_loop
-            .borrow()
-            .schedule_now(move || {
-                if let Some(s) = weak.upgrade() {
-                    unsafe {
-                        s.start_drag_internal(request);
+        if let Some(context) = self.context.get() {
+            context
+                .run_loop
+                .borrow()
+                .schedule_now(move || {
+                    if let Some(s) = weak.upgrade() {
+                        unsafe {
+                            s.start_drag_internal(request);
+                        }
                     }
-                }
-            })
-            .detach();
+                })
+                .detach();
+        }
         Ok(())
     }
 
     fn serialize_drag_data(&self, mut data: DragData) -> HashMap<u32, Vec<u8>> {
         let mut res = HashMap::new();
-        for adapter in &self.context.options.custom_drag_data_adapters {
-            adapter.prepare_drag_data(&mut data.properties, &mut res);
+        if let Some(context) = self.context.get() {
+            for adapter in &context.options.custom_drag_data_adapters {
+                adapter.prepare_drag_data(&mut data.properties, &mut res);
+            }
         }
         for adapter in &self.data_adapters {
             adapter.prepare_drag_data(&mut data.properties, &mut res);
@@ -103,8 +106,10 @@ impl DragContext {
     fn deserialize_drag_data(&self, data: IDataObject) -> DragData {
         let mut res: DragData = Default::default();
 
-        for adapter in &self.context.options.custom_drag_data_adapters {
-            adapter.retrieve_drag_data(data.clone(), &mut res.properties);
+        if let Some(context) = self.context.get() {
+            for adapter in &context.options.custom_drag_data_adapters {
+                adapter.retrieve_drag_data(data.clone(), &mut res.properties);
+            }
         }
 
         for adapter in &self.data_adapters {
@@ -114,6 +119,34 @@ impl DragContext {
         res
     }
 
+    pub unsafe fn do_drag_drop<'a>(
+        pdataobj: impl ::windows::IntoParam<'a, IDataObject>,
+        pdropsource: impl ::windows::IntoParam<'a, IDropSource>,
+        dwokeffects: u32,
+        pdweffect: *mut u32,
+    ) -> HRESULT {
+        #[cfg(windows)]
+        {
+            #[link(name = "OLE32")]
+            extern "system" {
+                fn DoDragDrop(
+                    pdataobj: ::windows::RawPtr,
+                    pdropsource: ::windows::RawPtr,
+                    dwokeffects: u32,
+                    pdweffect: *mut u32,
+                ) -> ::windows::HRESULT;
+            }
+            DoDragDrop(
+                pdataobj.into_param().abi(),
+                pdropsource.into_param().abi(),
+                ::std::mem::transmute(dwokeffects),
+                ::std::mem::transmute(pdweffect),
+            )
+        }
+        #[cfg(not(windows))]
+        unimplemented!("Unsupported target OS");
+    }
+
     unsafe fn start_drag_internal(&self, request: DragRequest) {
         let window = self.window.upgrade().unwrap();
         let data = self.serialize_drag_data(request.data);
@@ -143,7 +176,8 @@ impl DragContext {
         let source = DropSource::new();
         let ok_effects = convert_drag_effects(&request.allowed_effects);
         let mut effects_out: u32 = 0;
-        let res = DoDragDrop(data, source, ok_effects, &mut effects_out as *mut u32);
+        // TODO: Remove the wrapper once https://github.com/microsoft/windows-rs/issues/922 is resolved
+        let res = Self::do_drag_drop(data, source, ok_effects, &mut effects_out as *mut u32);
 
         if let Some(delegate) = window.delegate() {
             let mut effect = DragEffect::None;
